/*

This tap generates test patterns to allow correct adjustment of display devices.
The first 2 patterns are for setting of brightness and contrast and the next one is for adjusting the colour
Following that are full fields patterns for use with a Colorfacts or similar colour calibrator.
Note these Colorfacts patterns are full fields so may not be reliable with CRT or Plasma devices

The channel up and down keys cycle through the patterns and the Exit key terminates the Tap


Acknowledgement : Some of these patterns are based on those from Peter Finzels test disk which is highly recommended
for DVD player setup (www.peterfinzel.de)

History:
V1.0 colin             	    2004-07-24	initial version
V1.1 Kari Tiirikainen (KTi) 2009-01-11	Added secondary colors (yellow,cyan and magenta) and IRE level displays for greyscale patterns.
																				Also corrected greyscale pattern display so that also 100% IRE is shown.
V1.1 xyzzyx		    					2009-08-18  adjustments for TMS (sorry for wrong version numbering)
V1.2 xyzzyx		    					2009-08-26  corrected to new version number 1.2 and implemented changes of KTi's Version 1.1
V1.2a xyzzyx	    					2010-08-15  added support for TAPtoDate
V1.2b xyzzyx								2010-11-18	added FireBirdLib and KeyTranslate

*/

#include "tap.h"
// xyzzyx 101118
#ifdef _TMS_
#include "libFireBird.h"   // tested with FireBirdLib version "2010-11-14"
#endif

#define ID_testpatterns 0x80001408
#define PGM_NAME "Testpatterns" //xyzzyx 100815
#define PGM_VERS "V1.2b" //xyzzyx 100815
#define PGM_FULL PGM_NAME " " PGM_VERS //xyzzyx 100815

TAP_ID( 0x81AFFE02 ); //xyzzyx 100815 changed
TAP_PROGRAM_NAME(PGM_FULL); //xyzzyx 100815 changed
TAP_PROGRAM_VERSION(PGM_VERS); //xyzzyx 100815
TAP_AUTHOR_NAME("xyzzyx (Colin/KTi)");
TAP_DESCRIPTION("Testpatterns");
TAP_ETCINFO(__DATE__);

#define maxpagenumber 21
bool _exitFlag = FALSE;
word rgn;
word pagenumber = 1;
word windownumber = 0;


// xyzzyx 090818
//..............................Prototypes...............................
// dword TAP_EventHandler( word event, dword param1, dword param2 )
static int OpenPage(void);
static int ColourRamp(void);
static int Colorfacts (void);
static int Brightness (void);
static int Filterbars (void);
// int TAP_Main(void)

//.............................Event Handler.............................

dword TAP_EventHandler( word event, dword param1, dword param2 )
{

// xyzzyx 090818
#ifdef _TMS_
	UNUSED(param2);
#endif

	if( event == EVT_KEY )
    {
    	switch (param1)
		{
	  	case RKEY_ChUp:
 				pagenumber++;
				OpenPage();
				break;

			case RKEY_ChDown:
				pagenumber--;
				OpenPage();
				break;

	  	case RKEY_Exit:
				{
					_exitFlag = TRUE;
					return 0;
				}
		}


	}

    return event;
}


//.......................Page Selector....................................

static int OpenPage(void)

{

	if (pagenumber > maxpagenumber) pagenumber = 1;
	if (pagenumber < 1) pagenumber = maxpagenumber;

	if (pagenumber <=3)
	{
		switch (pagenumber)
		{
		case 1:
			ColourRamp();
			break;
		case 2:
			Brightness();
			break;
		case 3:
			Filterbars ();
			break;
		}
	}

	else
	{
		windownumber = pagenumber - 3;
		Colorfacts ();
	}
// xyzzyx 090818
#ifdef _TMS_
  TAP_Osd_Sync();
	return 1;
#else
  return 0;
#endif
}


//......................Display 7 Colour Ramp .............................


static int ColourRamp(void)

{
	word level = 0;
	dword height = 65;
	dword width  = 2;

	TAP_Osd_FillBox( rgn, 0 ,0, 720, 576, RGB(0,0,0));

// xyzzyx 090818
#ifdef _TMS_
	for (level=1;level<256;level++)
	{
		TAP_Osd_FillBox( rgn, (((level-1)*2)+104) , 25,  width,height,RGB(level,level,level));
		TAP_Osd_FillBox( rgn, (((level-1)*2)+104) , 100, width,height,RGB(level,0,0));
		TAP_Osd_FillBox( rgn, (((level-1)*2)+104) , 175, width,height,RGB(0,level,0));
		TAP_Osd_FillBox( rgn, (((level-1)*2)+104) , 250, width,height,RGB(0,0,level));
		TAP_Osd_FillBox( rgn, (((level-1)*2)+104) , 325, width,height,RGB(level,level,0));
		TAP_Osd_FillBox( rgn, (((level-1)*2)+104) , 400, width,height,RGB(level,0,level));
		TAP_Osd_FillBox( rgn, (((level-1)*2)+104) , 475, width,height,RGB(0,level,level));
	}
#else
	for (level=1;level<32;level++)
	{
		TAP_Osd_FillBox( rgn, (((level-1)*20)+40) , 25,  width,height,RGB(level,level,level));
		TAP_Osd_FillBox( rgn, (((level-1)*20)+40) , 100, width,height,RGB(level,0,0));
		TAP_Osd_FillBox( rgn, (((level-1)*20)+40) , 175, width,height,RGB(0,level,0));
		TAP_Osd_FillBox( rgn, (((level-1)*20)+40) , 250, width,height,RGB(0,0,level));
		TAP_Osd_FillBox( rgn, (((level-1)*20)+40) , 325, width,height,RGB(level,level,0));
		TAP_Osd_FillBox( rgn, (((level-1)*20)+40) , 400, width,height,RGB(level,0,level));
		TAP_Osd_FillBox( rgn, (((level-1)*20)+40) , 475, width,height,RGB(0,level,level));
	}
#endif

// xyzzyx 090818
#ifdef _TMS_
	return 1;
#else
  return 0;
#endif
}


// .............................Display Colorfacts patterns.......................

static int Colorfacts (void)

{
// xyzzyx 090818
#ifdef _TMS_
	dword colour = 0;
#else
	word colour = 0;
#endif
	char wlstr[4];

// xyzzyx 090818
#ifdef _TMS_
	switch (windownumber)
	{
		case 1:
			colour = COLOR_Red;
			break;
		case 2:
			colour = COLOR_Green;
			break;
		case 3:
			colour = COLOR_Blue;
			break;
		case 4:
			colour = COLOR_Yellow;
			break;
		case 5:
			colour = COLOR_Cyan;
			break;
		case 6:
			colour = COLOR_Magenta;
			break;
		case 7:
			colour = COLOR_White;
			break;
		case 8:
			colour = COLOR_Black;
			sprintf (wlstr,"%i%%",0);
			break;
		case 9:
			colour = RGB(26,26,26);
			sprintf (wlstr,"%i%%",10);
			break;
		case 10:
			colour = RGB(51,51,51);
			sprintf (wlstr,"%i%%",20);
			break;
		case 11:
			colour = RGB(77,77,77);
			sprintf (wlstr,"%i%%",30);
			break;
		case 12:
			colour = RGB(102,102,102);
			sprintf (wlstr,"%i%%",40);
			break;
		case 13:
			colour = RGB(128,128,128);
			sprintf (wlstr,"%i%%",50);
			break;
		case 14:
			colour = RGB(153,153,153);
			sprintf (wlstr,"%i%%",60);
			break;
		case 15:
			colour = RGB(179,179,179);
			sprintf (wlstr,"%i%%",70);
			break;
		case 16:
			colour = RGB(204,204,204);
			sprintf (wlstr,"%i%%",80);
			break;
		case 17:
			colour = RGB(230,230,230);
			sprintf (wlstr,"%i%%",90);
			break;
		case 18:
			colour = RGB(255,255,255);
			sprintf (wlstr,"%i%%",100);
			break;
	}
#else
	switch (windownumber)
	{
		case 1: /*Red*/
			colour = RGB(31,0,0);
			break;
		case 2:/*Green*/
			colour = RGB(0,31,0);
			break;
		case 3: /*Blue*/
			colour = RGB(0,0,31);
			break;
		case 4: /*Yellow*/
			colour = RGB(31,31,0);
			break;
		case 5: /*Cyan*/
			colour = RGB(0,31,31);
			break;
		case 6: /*Magenta*/
			colour = RGB(31,0,31);
			break;
		case 7:
			colour = RGB(31,31,31);
			break;
		case 8:
			colour = RGB(0,0,0);
			sprintf (wlstr,"%i%%",0);
			break;
		case 9:
			colour = RGB(3,3,3);
			sprintf (wlstr,"%i%%",10);
			break;
		case 10:
			colour = RGB(6,6,6);
			sprintf (wlstr,"%i%%",20);
			break;
		case 11:
			colour = RGB(9,9,9);
			sprintf (wlstr,"%i%%",30);
			break;
		case 12:
			colour = RGB(12,12,12);
			sprintf (wlstr,"%i%%",40);
			break;
		case 13:
			colour = RGB(16,16,16);
			sprintf (wlstr,"%i%%",50);
			break;
		case 14:
			colour = RGB(19,19,19);
			sprintf (wlstr,"%i%%",60);
			break;
		case 15:
			colour = RGB(22,22,22);
			sprintf (wlstr,"%i%%",70);
			break;
		case 16:
			colour = RGB(25,25,25);
			sprintf (wlstr,"%i%%",80);
			break;
		case 17:
			colour = RGB(28,28,28);
			sprintf (wlstr,"%i%%",90);
			break;
		case 18:
			colour = RGB(31,31,31);
			sprintf (wlstr,"%i%%",100);
			break;
	}
#endif

	TAP_Osd_FillBox( rgn, 0, 0, 720, 576, colour);

	if(windownumber>=8)
// xyzzyx 090826
#ifdef _TMS_
	    TAP_Osd_PutS(rgn,50,50,400,wlstr,RGB(255,0,0),0,0,FNT_Size_1419,0,ALIGN_LEFT);
#else
	    TAP_Osd_PutS(rgn,50,50,400,wlstr,RGB(31,0,0),0,0,FNT_Size_1419,0,ALIGN_LEFT);
#endif

// xyzzyx 090818
#ifdef _TMS_
	return 1;
#else
  return 0;
#endif
}

//.....................Brightness and contrast pattern........................

static int Brightness (void)

{
	word Level = 0;
	word height = 21 ;
	char levelstr[10];

// xyzzyx 090818
#ifdef _TMS_
	#define textcolour1 RGB(50,50,50)
	#define textcolour2 RGB(206,206,206)

	TAP_Osd_FillBox( rgn, 0 ,0, 720, 288,RGB(16,16,16));
	TAP_Osd_FillBox( rgn, 0 ,288, 720, 288,RGB(238,238,238));

	TAP_Osd_FillBox( rgn, 80 ,0, 4, 288,RGB(255,255,255));
	TAP_Osd_FillBox( rgn, 80 ,288, 4, 288,RGB(0,0,0));

	TAP_Osd_FillBox( rgn, 640 ,0, 4, 288,RGB(255,255,255));
	TAP_Osd_FillBox( rgn, 640 ,288, 4, 288,RGB(0,0,0));


	for (Level = 0; Level <=4; Level ++ )

	{
		TAP_Osd_FillBox( rgn, 300 ,((Level*height)+90), 120,height,RGB(Level*255/31,Level*255/31,Level*255/31));
		TAP_Osd_FillBox( rgn, 300 ,((Level*height)+90+288), 120,height,RGB((31-Level)*255/31,(31-Level)*255/31,(31-Level)*255/31));
		sprintf (levelstr,"%i",Level*255/31);

		TAP_Osd_PutS( rgn, 310 ,((Level*height)+90), 400,levelstr,textcolour1,0,0,FNT_Size_1419,0,ALIGN_LEFT);
		sprintf (levelstr,"%i",(31-Level)*255/31);
		TAP_Osd_PutS( rgn, 310 ,((Level*height)+90+288), 400,levelstr,textcolour2,0,0,FNT_Size_1419,0,ALIGN_LEFT);
	}

#else

	#define textcolour1 RGB(6,6,6)
	#define textcolour2 RGB(25,25,25)

	TAP_Osd_FillBox( rgn, 0 ,0, 720, 288,RGB(2,2,2));
	TAP_Osd_FillBox( rgn, 0 ,288, 720, 288,RGB(29,29,29));

	TAP_Osd_FillBox( rgn, 80 ,0, 4, 288,RGB(31,31,31));
	TAP_Osd_FillBox( rgn, 80 ,288, 4, 288,RGB(0,0,0));

	TAP_Osd_FillBox( rgn, 640 ,0, 4, 288,RGB(31,31,31));
	TAP_Osd_FillBox( rgn, 640 ,288, 4, 288,RGB(0,0,0));


	for (Level = 0; Level <=4; Level ++ )

	{
		TAP_Osd_FillBox( rgn, 300 ,((Level*height)+90), 120,height,RGB(Level,Level,Level));
		TAP_Osd_FillBox( rgn, 300 ,((Level*height)+90+288), 120,height,RGB(31-Level,31-Level,31-Level));
		sprintf (levelstr,"%i",Level);

		TAP_Osd_PutS( rgn, 310 ,((Level*height)+90), 400,levelstr,textcolour1,0,0,FNT_Size_1419,0,ALIGN_LEFT);
		sprintf (levelstr,"%i",(31-Level));
		TAP_Osd_PutS( rgn, 310 ,((Level*height)+90+288), 400,levelstr,textcolour2,0,0,FNT_Size_1419,0,ALIGN_LEFT);
	}
#endif


// xyzzyx 090818
#ifdef _TMS_
	return 1;
#else
  return 0;
#endif
}

//.....................Display Blue Cyan and Magenta bars for colour adjustment

static int Filterbars (void)

{
// xyzzyx 090818
#ifdef _TMS_
	TAP_Osd_FillBox( rgn, 0 ,0, 240, 288,RGB(0,0,255));
	TAP_Osd_FillBox( rgn, 0 ,288, 240, 288,RGB(0,255,255));
	TAP_Osd_FillBox( rgn, 240 ,0, 240, 576,RGB(255,0,255));
	TAP_Osd_FillBox( rgn, 480 ,0, 240, 288,RGB(0,255,255));
	TAP_Osd_FillBox( rgn, 480 ,288, 240, 288,RGB(0,0,255));
#else
	TAP_Osd_FillBox( rgn, 0 ,0, 240, 288,RGB(0,0,31));
	TAP_Osd_FillBox( rgn, 0 ,288, 240, 288,RGB(0,31,31));
	TAP_Osd_FillBox( rgn, 240 ,0, 240, 576,RGB(31,0,31));
	TAP_Osd_FillBox( rgn, 480 ,0, 240, 288,RGB(0,31,31));
	TAP_Osd_FillBox( rgn, 480 ,288, 240, 288,RGB(0,0,31));
#endif

// xyzzyx 090818
#ifdef _TMS_
	return 1;
#else
  return 0;
#endif
}




//............................TAP_Main.........................................


int TAP_Main(void)
{
    int tvRadio, svcNum;

// xyzzyx 101118
#ifdef _TMS_
		KeyTranslate(TRUE, &TAP_EventHandler);
#endif

    pagenumber = 1;

    TAP_Channel_GetCurrent( &tvRadio, &svcNum );
    TAP_Channel_Stop( 1 );
    rgn = TAP_Osd_Create( 0, 0, 720, 576, 0, FALSE );

    ColourRamp();

// xyzzyx 090818
#ifdef _TMS_
		TAP_Osd_Sync();
#endif

    while( _exitFlag == FALSE )
    {
      TAP_SystemProc();
    }
    TAP_Osd_Delete( rgn );
    TAP_Channel_Start( 1, tvRadio, svcNum );
    return 0;
}

